--[[
This controls the configuration settings (and display of a sample set of
actions based on those settings) for each ActionSetLayout.  Each
BarActionSetLayout is associated with an BarActionSetLayoutConfig, and each
BarActionSetLayoutConfig refers to that same BarActionSetLayout instance.
--]]
uab.modules.BarLayoutManager.LayoutConfig = uab.mixins.aceOO.Class(IActionSetLayoutConfig, uab.mixins.aceEvent)
uab.modules.BarLayoutManager.LayoutConfig.count = 0

function uab.modules.BarLayoutManager.LayoutConfig.prototype:init(layout, settings)
    uab.modules.BarLayoutManager.LayoutConfig.super.prototype.init(self)

    self.settings = settings

    self:SettingsChanged()

    uab.modules.BarLayoutManager.LayoutConfig.count = uab.modules.BarLayoutManager.LayoutConfig.count + 1
    
    self.number = uab.modules.BarLayoutManager.LayoutConfig.count
    self.layout = layout
end

function uab.modules.BarLayoutManager.LayoutConfig.prototype:SettingsChanged(settings)
    if not self.settings then
        self.settings = {}
    end

    if not settings then
        settings = self.settings
    end
    
    self.settings.overflowClockwise = (settings and settings.overflowClockwise) or "true"
    self.settings.placement = (settings and settings.placement) or uab.constants.placement.right
    self.settings.growDirection = (settings and settings.growDirection) or uab.constants.grow.right
    self.settings.growLimit = (settings and settings.growLimit) or 4

    -- Make sure all the settings properly coordinate with one another
    
    self:SetOverflowClockwise(self.settings.overflowClockwise)
    self:SetPlacement(self.settings.placement)
    self:SetGrowDirection(self.settings.growDirection)
    self:SetGrowLimit(self.settings.growLimit)
end

function uab.modules.BarLayoutManager.LayoutConfig.prototype:UseDefaults(bypassNotification)
	if not self.settings then
	    self.settings = {}
	end

    self.settings.overflowClockwise = "true"
    self.settings.placement = uab.constants.placement.right
    self.settings.growDirection = uab.constants.grow.right
    self.settings.growLimit = 4

    self:SettingsChanged()
	
    if not bypassNotification then
        self.layout:LayoutSettingsChanged()
    end
end

function uab.modules.BarLayoutManager.LayoutConfig.prototype:MenuOptionArgs()
	--[[
	Build an AceOptions table of the options to be used to configure this
	config instance.  This table is inserted into a parent menu as a set of
	options.  If there are several options, just define them at the top level
	of the returned table.  Do not create a 'group' at the top level to enclose
	the options.
	--]]
	
	local options = {
        growlimit = {
            type = 'range',
            name = "Grow Limit",
            desc = "Maximum number of buttons to place in each row before expanding to the next row",
            min = 1,
            max = 24,
            step = 1,
            isPercent = false,
            get = function()
                return self.settings.growLimit
            end,
            set = function(limit)
                self:SetGrowLimit(limit)
            end,
        },
        clockwise = {
            type = 'toggle',
            name = "Overflow clockwise",
            desc = "When a bar grows past the grow limit, add another row relative to the last bar in a clockwise direction",
            disabled = function() return not self.cansetdirection end,
            get = function()
                return self.settings.overflowClockwise == "true"
            end,
            set = function(v)
                if v then
					self:SetOverflowClockwise("true")
				else
				    self:SetOverflowClockwise("false")
                end
            end,
        },
        placement = {
            type = 'text',
            name = "First button placement",
            desc = "Placement of the first button, relative to the cancel button",
            get = function()
                return uab.constants.PlacementDisplayValue(self.settings.placement)
            end,
            set = function(placement)
                if placement == uab.constants.PlacementDisplayValue(uab.constants.placement.top) then
                    self:SetPlacement(uab.constants.placement.top)
				elseif placement == uab.constants.PlacementDisplayValue(uab.constants.placement.bottom) then
                	self:SetPlacement(uab.constants.placement.bottom)
				elseif placement == uab.constants.PlacementDisplayValue(uab.constants.placement.left) then
                	self:SetPlacement(uab.constants.placement.left)
				elseif placement == uab.constants.PlacementDisplayValue(uab.constants.placement.right) then
                	self:SetPlacement(uab.constants.placement.right)
				end
            end,

            validate = {
				uab.constants.PlacementDisplayValue(uab.constants.placement.top),
				uab.constants.PlacementDisplayValue(uab.constants.placement.bottom),
				uab.constants.PlacementDisplayValue(uab.constants.placement.left),
				uab.constants.PlacementDisplayValue(uab.constants.placement.right),
			},
        },
        direction = {
            type = 'text',
            name = "Grow direction",
            desc = "The direction in which new buttons are added in each row",
            get = function()
                return uab.constants.GrowDirectionDisplayValue(self.settings.growDirection)
            end,
            set = function(direction)
                if direction == uab.constants.GrowDirectionDisplayValue(uab.constants.grow.up) then
	                self:SetGrowDirection(uab.constants.grow.up)
				elseif direction == uab.constants.GrowDirectionDisplayValue(uab.constants.grow.down) then
	                self:SetGrowDirection(uab.constants.grow.down)
				elseif direction == uab.constants.GrowDirectionDisplayValue(uab.constants.grow.left) then
	                self:SetGrowDirection(uab.constants.grow.left)
				elseif direction == uab.constants.GrowDirectionDisplayValue(uab.constants.grow.right) then
	                self:SetGrowDirection(uab.constants.grow.right)
				end
            end,

            validate = {
				uab.constants.GrowDirectionDisplayValue(uab.constants.grow.up),
				uab.constants.GrowDirectionDisplayValue(uab.constants.grow.down),
				uab.constants.GrowDirectionDisplayValue(uab.constants.grow.left),
				uab.constants.GrowDirectionDisplayValue(uab.constants.grow.right),
			},
        },
	}
	
	return options
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.modules.BarLayoutManager.LayoutConfig.prototype:SetPlacement(value)
    local notify = (value ~= self.settings.placement)

    self.settings.placement = value

    self:SynchronizeGrowPattern()

    if notify then
        self.layout:LayoutSettingsChanged()
    end
end

function uab.modules.BarLayoutManager.LayoutConfig.prototype:SetGrowDirection(value)
    local notify = (value ~= self.settings.growDirection)

    self.settings.growDirection = value

    self:SynchronizeGrowPattern()

    if notify then
        self.layout:LayoutSettingsChanged()
    end
end

function uab.modules.BarLayoutManager.LayoutConfig.prototype:SetGrowLimit(value)
    local notify = (value ~= self.settings.growLimit)

    self.settings.growLimit = value
    
    if notify then
        self.layout:LayoutSettingsChanged()
    end
end

function uab.modules.BarLayoutManager.LayoutConfig.prototype:SetOverflowClockwise(value)
    local newval

    if value and (tonumber(value) == 1 or value == "true") then
        newval = "true"
    else
        newval = "false"
    end

    local notify = (newval ~= self.settings.overflowClockwise)

    self.settings.overflowClockwise = newval
    
    if notify then
        self.layout:LayoutSettingsChanged()
    end
end

function uab.modules.BarLayoutManager.LayoutConfig.prototype:EnableOverflowDirection()
    self.cansetdirection = true
end
    	
function uab.modules.BarLayoutManager.LayoutConfig.prototype:DisableOverflowDirection()
    self.cansetdirection = nil
end
    	
function uab.modules.BarLayoutManager.LayoutConfig.prototype:SynchronizeGrowPattern()
    self:EnableOverflowDirection()
    	
    if self.settings.placement and self.settings.growDirection then
        if self.settings.placement == uab.constants.placement.top then
            if self.settings.growDirection == uab.constants.grow.down then
                self:SetGrowDirection(uab.constants.grow.up)
            elseif self.settings.growDirection == uab.constants.grow.right then
                self:SetOverflowClockwise("false")
                self:DisableOverflowDirection()
            elseif self.settings.growDirection == uab.constants.grow.left then
                self:SetOverflowClockwise("true")
                self:DisableOverflowDirection()
            end
        elseif self.settings.placement == uab.constants.placement.bottom then
            if self.settings.growDirection == uab.constants.grow.up then
                self:SetGrowDirection(uab.constants.grow.down)
            elseif self.settings.growDirection == uab.constants.grow.right then
                self:SetOverflowClockwise("true")
                self:DisableOverflowDirection()
            elseif self.settings.growDirection == uab.constants.grow.left then
                self:SetOverflowClockwise("false")
                self:DisableOverflowDirection()
            end
        elseif self.settings.placement == uab.constants.placement.left then
            if self.settings.growDirection == uab.constants.grow.right then
                self:SetGrowDirection(uab.constants.grow.left)
            elseif self.settings.growDirection == uab.constants.grow.up then
                self:SetOverflowClockwise("false")
                self:DisableOverflowDirection()
            elseif self.settings.growDirection == uab.constants.grow.down then
                self:SetOverflowClockwise("true")
                self:DisableOverflowDirection()
            end
        elseif self.settings.placement == uab.constants.placement.right then
            if self.settings.growDirection == uab.constants.grow.left then
                self:SetGrowDirection(uab.constants.grow.right)
            elseif self.settings.growDirection == uab.constants.grow.up then
                self:SetOverflowClockwise("true")
                self:DisableOverflowDirection()
            elseif self.settings.growDirection == uab.constants.grow.down then
                self:SetOverflowClockwise("false")
                self:DisableOverflowDirection()
            end
        end
    end
end

